/* $Id: et6000_text16.h,v 1.3 1998/11/24 18:45:21 marcus Exp $
******************************************************************************

   Tseng Labs ET6000 hardware text mode manipulation prototypes

   Copyright (C) 1997           Jim Ursetto     [jim.ursetto@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/


#ifndef _ET6000_TEXT16_H
#define _ET6000_TEXT16_H

#include <kgi/system.h>
#include <IBM/vga_text16.h>

/* Aaahhh, red cursor.  Not that icky yellow one. :) */ 
#define CURSOR_MASK	0x48	/* Attribute mask (bg,fg) */
#define POINTER_MASK	0xFF	/* Attribute mask (bg,fg) */

/* !! FIXME */
#define CURSOR_HW
#define PCELLSIDEX	3	/* Horizontal length of pointer cell box */
#define PCELLSIDEY	3	/* Vertical length of pointer cell box */

#define PCELLTOTAL	(PCELLSIDEX * PCELLSIDEY)		
#define PCELLOF(fp, xsize, i) ((fp) + (((i) / PCELLSIDEX) * (xsize)) + ((i) % PCELLSIDEX))

typedef struct text16_point_s
{
	int	visible;	/* Should I visible the old data? */
	int	offset;		/* Offset of old point (charcells) */
	int	offset2;	/* Extra offset, due to split line */
	int	splity;         /* Split line in old point (0..PCELLSIDEY) */
	kgi_uint	maxx;		/* Max visible x of old point (charcells) */
	kgi_uint	maxy;		/* Max visible y of old point (charcells) */
	int bitoffset;	/* Offset of old point (dots) */
	charcell data[PCELLTOTAL];	/* Data at old point */
} text16_point;
	
struct text16_data_s
{
	unsigned int magic;
	int		font_changed;
	unsigned long	origin;
	int		splitline;
	text16_point  	pointer;
#ifndef CURSOR_HW
	text16_point	cursor;
#endif
} text16_data[MAX_NR_DISPLAYS];

#define ORIGIN 		(data->origin)
#define SPLITLINE	(data->splitline)
#define FONTCHANGED	(data->font_changed)

#define PVISIBLE	(data->pointer.visible)
#define POFFSET1	(data->pointer.offset)
#define POFFSET2	(data->pointer.offset2)
#define PSPLITY		(data->pointer.splity)
#define PMAXX		(data->pointer.maxx)
#define PMAXY		(data->pointer.maxy)
#define PBITOFFSET	(data->pointer.bitoffset)
#define PDATA		(data->pointer.data)

#ifndef CURSOR_HW
#define CVISIBLE	(data->cursor.visible)
#define COFFSET		(data->cursor.offset)
#define CDATA		(data->cursor.data[0])
#endif

/* The text16_data structure resided under dpy->driver_data originally.  I
 * moved it to dpy->driver_data->driver so it doesn't clobber any other private
 * data (chipset, for example).  This change has been propagated to the VGA
 * driver.
 */

#define GET_TEXT_DATA struct text16_data_s *data = \
		(struct text16_data_s *)KGIM_PRIVATE(dpy, driver)

void text16_data_init(void);
struct text16_data_s *text16_data_realloc(struct text16_data_s *old);
void text16_set_font(struct kgi_display *dpy, struct kgi_font *font,
                     uint from, uint to);
void text16_set_origin(struct kgi_display *dpy, uint x, uint y);
void text16_set_split(struct kgi_display *dpy, int line);
void text16_show_pointer(struct kgi_display *dpy, uint x, uint y);
void text16_hide_pointer(struct kgi_display *dpy);
void text16_show_cursor(struct kgi_display *dpy, uint x, uint y);
void text16_hide_cursor(struct kgi_display *dpy);
#define text16_undo_cursor      text16_hide_cursor
#define text16_undo_pointer     text16_hide_pointer
#define text16_set_frame    NULL 

#endif /* _ET6000_TEXT16_H */

